/** @file
 *  PKCS11 ESMART Token GOST library.
 *
 *  Description: gost management definitions.
 *
 *  Copyright 2017 by ISBC Group
 *  All rights reserved.
 */
#pragma once

#define NSSCK_VENDOR_PKCS11_RU_TEAM     (CKR_VENDOR_DEFINED | 0x54321000)
#define CK_VENDOR_PKCS11_RU_TEAM_TC26   NSSCK_VENDOR_PKCS11_RU_TEAM

/* GOST KEY TYPES */
#define CKK_GOSTR3410                   0x00000030
#define CKK_GOSTR3411                   0x00000031
#define CKK_GOST28147                   0x00000032
#define CKK_GOSTR3410_256               CKK_GOSTR3410
#define CKK_GOSTR3410_512               (CK_VENDOR_PKCS11_RU_TEAM_TC26 |0x003)
#define CKK_KUZNECHIK                   (CK_VENDOR_PKCS11_RU_TEAM_TC26 |0x004)

/* GOST OBJECT ATTRIBUTES */
#define CKA_GOSTR3410PARAMS             0x00000250
#define CKA_GOSTR3411PARAMS             0x00000251
#define CKA_GOST28147_PARAMS			0x00000252
#define CKA_GOSTR3410_256PARAMS         CKA_GOSTR3410PARAMS
#define CKA_GOSTR3410_512PARAMS         (CK_VENDOR_PKCS11_RU_TEAM_TC26 |0x004)

/* PKCS #5 PRF Functions */
#define CKP_PKCS5_PBKD2_HMAC_GOSTR3411_2012_256 (CK_VENDOR_PKCS11_RU_TEAM_TC26 |0x002)
#define CKP_PKCS5_PBKD2_HMAC_GOSTR3411_2012_512 (CK_VENDOR_PKCS11_RU_TEAM_TC26 |0x003)

/* GOST MECHANISMS */
#define CKM_GOSTR3410_KEY_PAIR_GEN		0x00001200
#define CKM_GOSTR3410					0x00001201
#define CKM_GOSTR3410_WITH_GOSTR3411	0x00001202
#define CKM_GOSTR3410_KEY_WRAP			0x00001203
#define CKM_GOSTR3410_DERIVE			0x00001204
#define CKM_GOSTR3411					0x00001210
#define CKM_GOSTR3411_HMAC				0x00001211
#define CKM_GOST28147_KEY_GEN			0x00001220
#define CKM_GOST28147_ECB				0x00001221
#define CKM_GOST28147					0x00001222
#define CKM_GOST28147_MAC				0x00001223
#define CKM_GOST28147_KEY_WRAP			0x00001224

#define CKM_GOSTR3410_256_KEY_PAIR_GEN          CKM_GOSTR3410_KEY_PAIR_GEN
#define CKM_GOSTR3410_512_KEY_PAIR_GEN          (CK_VENDOR_PKCS11_RU_TEAM_TC26 |0x005)
#define CKM_GOSTR3410_256                       CKM_GOSTR3410
#define CKM_GOSTR3410_512                       (CK_VENDOR_PKCS11_RU_TEAM_TC26 |0x006)
#define CKM_GOSTR3410_2012_DERIVE               (CK_VENDOR_PKCS11_RU_TEAM_TC26 |0x007)
#define CKM_GOSTR3410_12_DERIVE                 CKM_GOSTR3410_2012_DERIVE
#define CKM_GOSTR3410_WITH_GOSTR3411_94         CKM_GOSTR3410_WITH_GOSTR3411
#define CKM_GOSTR3410_WITH_GOSTR3411_2012_256   (CK_VENDOR_PKCS11_RU_TEAM_TC26 |0x008)
#define CKM_GOSTR3410_WITH_GOSTR3411_2012_512   (CK_VENDOR_PKCS11_RU_TEAM_TC26 |0x009)
#define CKM_GOSTR3410_WITH_GOSTR3411_12_256     CKM_GOSTR3410_WITH_GOSTR3411_2012_256
#define CKM_GOSTR3410_WITH_GOSTR3411_12_512     CKM_GOSTR3410_WITH_GOSTR3411_2012_512
#define CKM_GOSTR3410_PUBLIC_KEY_DERIVE         (CK_VENDOR_PKCS11_RU_TEAM_TC26 |0x00A)

#define CKM_GOSTR3411_94                        CKM_GOSTR3411
#define CKM_GOSTR3411_2012_256                  (CK_VENDOR_PKCS11_RU_TEAM_TC26 |0x012)
#define CKM_GOSTR3411_2012_512                  (CK_VENDOR_PKCS11_RU_TEAM_TC26 |0x013)
#define CKM_GOSTR3411_12_256                    CKM_GOSTR3411_2012_256
#define CKM_GOSTR3411_12_512                    CKM_GOSTR3411_2012_512
#define CKM_GOSTR3411_94_HMAC                   CKM_GOSTR3411_HMAC
#define CKM_GOSTR3411_2012_256_HMAC             (CK_VENDOR_PKCS11_RU_TEAM_TC26 |0x014)
#define CKM_GOSTR3411_2012_512_HMAC             (CK_VENDOR_PKCS11_RU_TEAM_TC26 |0x015)
#define CKM_GOSTR3411_12_256_HMAC               CKM_GOSTR3411_2012_256_HMAC
#define CKM_GOSTR3411_12_512_HMAC               CKM_GOSTR3411_2012_512_HMAC
#define CKM_TLS_GOST_PRF_2012_256               (CK_VENDOR_PKCS11_RU_TEAM_TC26 |0x016)
#define CKM_TLS_GOST_PRF_2012_512               (CK_VENDOR_PKCS11_RU_TEAM_TC26 |0x017)
#define CKM_TLS_GOST_PRE_MASTER_KEY_GEN         CKM_GOST28147_KEY_GEN
#define CKM_TLS_GOST_MASTER_KEY_DERIVE_2012_256 (CK_VENDOR_PKCS11_RU_TEAM_TC26 |0x018)

#define CKM_KUZNECHIK_KEY_GEN                   (CK_VENDOR_PKCS11_RU_TEAM_TC26 |0x019)
#define CKM_KUZNECHIK_ECB                       (CK_VENDOR_PKCS11_RU_TEAM_TC26 |0x01A)
#define CKM_KUZNECHIK_CTR                       (CK_VENDOR_PKCS11_RU_TEAM_TC26 |0x01B)
#define CKM_KUZNECHIK_CFB                       (CK_VENDOR_PKCS11_RU_TEAM_TC26 |0x01C)
#define CKM_KUZNECHIK_OFB                       (CK_VENDOR_PKCS11_RU_TEAM_TC26 |0x01D)
#define CKM_KUZNECHIK_CBC                       (CK_VENDOR_PKCS11_RU_TEAM_TC26 |0x01E)
#define CKM_KUZNECHIK_MAC                       (CK_VENDOR_PKCS11_RU_TEAM_TC26 |0x01F)
#define CKM_MAGMA_ECB                           CKM_GOST28147_ECB
#define CKM_MAGMA_CTR                           (CK_VENDOR_PKCS11_RU_TEAM_TC26 |0x020)
#define CKM_MAGMA_CFB                           (CK_VENDOR_PKCS11_RU_TEAM_TC26 |0x021)
#define CKM_MAGMA_OFB                           (CK_VENDOR_PKCS11_RU_TEAM_TC26 |0x022)
#define CKM_MAGMA_CBC                           (CK_VENDOR_PKCS11_RU_TEAM_TC26 |0x023)
#define CKM_MAGMA_MAC                           (CK_VENDOR_PKCS11_RU_TEAM_TC26 |0x024)
#define CKM_KDF_4357                            (CK_VENDOR_PKCS11_RU_TEAM_TC26 |0x025)
#define CKM_KDF_GOSTR3411_2012_256              (CK_VENDOR_PKCS11_RU_TEAM_TC26 |0x026)


/* Alternative */
#define CKA_GOSTR3410_PARAMS                    CKA_GOSTR3410PARAMS
#define CKA_GOSTR3411_PARAMS                    CKA_GOSTR3411PARAMS
#define CKA_GOSTR3410_256_PARAMS                CKA_GOSTR3410_256PARAMS
#define CKM_GOSTR3411_12_256                    CKM_GOSTR3411_2012_256
#define CKM_GOSTR3410_WITH_GOSTR3411_12_256     CKM_GOSTR3410_WITH_GOSTR3411_2012_256
#define CKM_GOSTR3410_12_DERIVE                 CKM_GOSTR3410_2012_DERIVE

namespace oid
{
extern const BYTE id_Gost28147_89_CryptoPro_A_ParamSet				[];
extern const BYTE id_Gost28147_89_CryptoPro_B_ParamSet				[];
extern const BYTE id_Gost28147_89_CryptoPro_C_ParamSet				[];
extern const BYTE id_Gost28147_89_CryptoPro_D_ParamSet				[];
extern const BYTE id_Gost28147_89_CryptoPro_RIC_1_ParamSet			[];

extern const BYTE id_GostR3410_2001_TestParamSet					[];
extern const BYTE id_GostR3410_2001_CryptoPro_A_ParamSet			[];
extern const BYTE id_GostR3410_2001_CryptoPro_B_ParamSet			[];
extern const BYTE id_GostR3410_2001_CryptoPro_C_ParamSet			[];
extern const BYTE id_GostR3410_2001_CryptoPro_XchA_ParamSet			[];
extern const BYTE id_GostR3410_2001_CryptoPro_XchB_ParamSet			[];

extern const BYTE id_GostR3411_94_TestParamSet						[];
extern const BYTE id_GostR3411_94_CryptoProParamSet					[];
extern const BYTE id_GostR3411_12_256_ParamSet						[];

extern const BYTE id_ecdsa_prime256v1										[];
}

bool CheckGostParamOid( BYTE bType, const BYTE *pOid, DWORD dwOidLen, BYTE *pbAddParam );
bool CheckEcdsaParamOid( BYTE bType, const BYTE *pOid, DWORD dwOidLen, BYTE *pbAddParam );
