#include <string>
#include <vector>

#ifdef _WINDOWS
	#include "stdafx.h"
#endif

#ifndef _WINDOWS
    #include <dlfcn.h>
#endif

#include <cryptoki.h>

#ifdef __APPLE__
    #include <PCSC/wintypes.h>
    #include <PCSC/winscard.h>
#elif _WINDOWS
#else
    #include <wintypes.h>
    #include <winscard.h>
    #include <reader.h>
#endif
#include <gost.h>

#ifndef _WINDOWS
	#include <dlfcn.h>
#endif
#include <iostream>
#include <iomanip>
#include <vector>
#include <string>
#include <sstream>
#include <string.h>

using namespace std;

#define FILL_ATTR(attr, typ, val, len) {(attr).type=(typ); (attr).pValue=(val); (attr).ulValueLen=len;} 

int main(int argc, char* argv[])
{
	cout << "Start with default PIN" << endl;
	BYTE pin[8];
	size_t pinLen = sizeof(pin);
	memcpy(pin, "12345678", pinLen);
	
	CK_FUNCTION_LIST_PTR p11 = NULL; /**< The PKCS #11 API functions pointers. */
	bool IsInitialized = false;
	
#ifdef _WINDOWS
    HMODULE pkcsLib;
#else
    void *pkcsLib;
#endif

	//////////////// Init library PKCS#11
#ifdef _WINDOWS
	pkcsLib = LoadLibrary("isbc_pkcs11_main.dll");
	if (!pkcsLib)
	{
		cout << "Error while load PKCS#11: "  << GetLastError() << endl;
		return 1;
	}
#else
	pkcsLib = dlopen("libisbc_pkcs11_main.so", RTLD_NOW);

	if (!pkcsLib)
	{
		cout << "Error while load PKCS#11: "  << dlerror() << endl;
		return 1;
	}
#endif
	
	void  (*pfoo)(CK_FUNCTION_LIST_PTR*);
	CK_RV rv;
	CK_ULONG ulSlotCount;
	
#ifdef _WINDOWS
	pfoo = (void (*)(CK_FUNCTION_LIST_PTR*)) GetProcAddress(pkcsLib, "C_GetFunctionList");
#else
	pfoo = (void (*)(CK_FUNCTION_LIST_PTR*)) dlsym(pkcsLib, "C_GetFunctionList");
#endif
	
	if(pfoo == NULL)
	{
		cout << "Can't get C_GetFunctionList symbol" << endl;
		return 2;
	}

	pfoo(&p11);
	if(p11 == NULL)
	{
		cout << "C_GetFunctionList error" << endl;
		return 3;
	}

	rv = p11->C_Initialize(NULL);
	if(rv == CKR_CRYPTOKI_ALREADY_INITIALIZED)
	{
		IsInitialized = true;
	}

	if (rv != CKR_OK)
	{
	    cout << "C_Initialize error. Code: " << rv << endl;
	    return 4;
	}

	rv = p11->C_GetSlotList(FALSE, NULL_PTR, &ulSlotCount);
	if (rv != CKR_OK)
	{
	    cout << "C_GetSlotList failed. Code: " << rv << endl;
	    return 5;
	}

	IsInitialized = true;

	// open session
	CK_SESSION_HANDLE	hSession;
	CK_SLOT_ID SlotId = 0;
	rv = p11->C_OpenSession(SlotId, CKF_SERIAL_SESSION | CKF_RW_SESSION, NULL, NULL, &hSession);

	if(rv != CKR_OK)
	{
		cout << "C_OpenSession failed. Code: " << rv << endl;
		return 6;
	}
	
	//Log-In the user
	rv = p11->C_Login(hSession, CKU_USER, pin, pinLen);
	if (rv != CKR_OK && rv != CKR_USER_ALREADY_LOGGED_IN)
	{
		p11->C_CloseSession(hSession);
		cout << "Login failed. Code: " << rv << endl;
		return 7;
	}
	
	// Import certificate from PKCS#12 (Buffer and BufferLen should be fill PKCS#12 data)
	// PassWord and PassWordLen should be fill by password of PKCS#12 container
	//BYTE *Buffer, CK_ULONG BufferLen, BYTE* PassWord, CK_ULONG PassWordLen;
	//rv = p11->C_ACS_LoadCert(hSession, Buffer, BufferLen, PassWord, PassWordLen);
	//if (rv != CKR_OK)
	//{
	//	p11->C_CloseSession(hSession);
	//	cout << "Load failed. Code: " << rv << endl;
	//	return 8;
	//}

	// generate GOST3410 key pair
	CK_OBJECT_HANDLE hPublicKey, hPrivateKey;
	CK_MECHANISM mechanism = {CKM_GOSTR3410_KEY_PAIR_GEN, NULL_PTR, 0};
	mechanism.mechanism = CKM_GOSTR3410_KEY_PAIR_GEN;
	CK_BBOOL _true = TRUE;
	CK_OBJECT_CLASS pubkey_class = CKO_PUBLIC_KEY;
	CK_OBJECT_CLASS privkey_class = CKO_PRIVATE_KEY;
	CK_ATTRIBUTE publicKeyTemplate[20] = {
		{CKA_CLASS, &pubkey_class, sizeof(pubkey_class)},
		{CKA_TOKEN, &_true, sizeof(_true)},
		{CKA_ENCRYPT, &_true, sizeof(_true)},
		{CKA_VERIFY, &_true, sizeof(_true)},
		{CKA_WRAP, &_true, sizeof(_true)}
	};
	int n_pubkey_attr = 7;
	CK_ATTRIBUTE privateKeyTemplate[20] = {
		{CKA_CLASS, &privkey_class, sizeof(privkey_class)},
		{CKA_TOKEN, &_true, sizeof(_true)},
		{CKA_PRIVATE, &_true, sizeof(_true)},
		{CKA_SENSITIVE, &_true, sizeof(_true)},
		{CKA_DECRYPT, &_true, sizeof(_true)},
		{CKA_SIGN, &_true, sizeof(_true)},
		{CKA_UNWRAP, &_true, sizeof(_true)}
	};
	int n_privkey_attr = 7;

	char label[] = "new label";
	BYTE id[] = "new id";
	CK_ULONG idLen;
	FILL_ATTR(publicKeyTemplate[n_pubkey_attr], CKA_LABEL, (CK_VOID_PTR) label, (CK_ULONG) strlen(label));
	FILL_ATTR(privateKeyTemplate[n_privkey_attr], CKA_LABEL, (CK_VOID_PTR) label, (CK_ULONG) strlen(label));
	n_pubkey_attr++;
	n_privkey_attr++;
		
	FILL_ATTR(publicKeyTemplate[n_pubkey_attr], CKA_ID, (CK_VOID_PTR) id, (CK_ULONG) strlen(id));
	FILL_ATTR(privateKeyTemplate[n_privkey_attr], CKA_ID, (CK_VOID_PTR) id, (CK_ULONG) strlen(id));
	n_pubkey_attr++;
	n_privkey_attr++;

	rv = p11->C_GenerateKeyPair(hSession, &mechanism, publicKeyTemplate, n_pubkey_attr,
		privateKeyTemplate, n_privkey_attr, &hPublicKey, &hPrivateKey); 
	
	p11->C_CloseSession(hSession);

	// Finalase library PKCS#11
	if (IsInitialized)
	{
		IsInitialized = false;

		if(p11 != NULL)
		{
			p11->C_Finalize(NULL);
		}
	}

	
	if (pkcsLib)
	{
#ifdef _WINDOWS
		FreeLibrary(pkcsLib);
//t		CloseHandle(pkcsLib);
#else
	    dlclose(pkcsLib);
#endif
	    pkcsLib = NULL;
	}

	return 0;
}
