//
//  Utils.hpp
//  EsmartTokenGost28147
//
//  Created by Sergey Panov on 28/02/2019.
//  Copyright © 2019 ISBC Group. All rights reserved.
//

#ifndef Utils_hpp
#define Utils_hpp

#ifndef _WINDOWS
#include <dlfcn.h>
#include <string.h>
#include <termios.h>
#include <unistd.h>
#endif

#include <cryptoki.h>

#ifdef __APPLE__
#include <PCSC/wintypes.h>
#include <PCSC/winscard.h>
#elif _WINDOWS
#include "stdafx.h"
#else
#include <wintypes.h>
#include <winscard.h>
#include <reader.h>
#endif

#include <string>
#include <vector>
#include <iostream>
#include <iomanip>
#include <sstream>
#include <fstream>
#include <sstream>
#include <time.h>
#include <algorithm>
#include <gost.h>

using namespace std;

CK_FUNCTION_LIST_PTR p11 = NULL;
bool IsInitialized = false;
CK_SESSION_HANDLE hSession = CK_INVALID_HANDLE;

#ifdef _WINDOWS
HMODULE pkcsLib;
#else
void *pkcsLib;
#endif

/**
 * cleanup allocated resources
 */
void cleanup()
{
    // Finalase library PKCS#11
    if (IsInitialized)
    {
        IsInitialized = false;
        if(p11 != NULL)
        {
            if (hSession != CK_INVALID_HANDLE)
            {
                p11->C_CloseSession(hSession);
                hSession = CK_INVALID_HANDLE;
            }
            
            p11->C_Finalize(NULL);
            p11 = NULL;
        }
    }
    
    if (pkcsLib)
    {
#ifdef _WINDOWS
        FreeLibrary(pkcsLib);
#else
        dlclose(pkcsLib);
#endif
        pkcsLib = NULL;
    }
}

/**
 * init all global variables
 */
int init() {
    //////////////// Init library PKCS#11
#ifdef _WINDOWS
    pkcsLib = LoadLibrary("isbc_pkcs11_main.dll");
    if (!pkcsLib)
    {
        cout << "Error while load PKCS#11: "  << GetLastError() << endl;
        cleanup();
        return 1;
    }
#elif __APPLE__
    pkcsLib = dlopen("libisbc_pkcs11_main.dylib", RTLD_NOW);
#else
    pkcsLib = dlopen("libisbc_pkcs11_main.so", RTLD_NOW);
    
    if (!pkcsLib)
    {
        cout << "Error while load PKCS#11: "  << dlerror() << endl;
        cleanup();
        return 1;
    }
#endif
    
    void  (*pfoo)(CK_FUNCTION_LIST_PTR*);
    
#ifdef _WINDOWS
    pfoo = (void (*)(CK_FUNCTION_LIST_PTR*)) GetProcAddress(pkcsLib, "C_GetFunctionList");
#else
    pfoo = (void (*)(CK_FUNCTION_LIST_PTR*)) dlsym(pkcsLib, "C_GetFunctionList");
#endif
    
    if(pfoo == NULL)
    {
        cout << "Can't get C_GetFunctionList symbol" << endl;
        cleanup();
        return 2;
    }
    
    pfoo(&p11);
    if(p11 == NULL)
    {
        cout << "C_GetFunctionList error" << endl;
        cleanup();
        return 3;
    }
    
    CK_RV rv = p11->C_Initialize(NULL);
    if(rv == CKR_CRYPTOKI_ALREADY_INITIALIZED)
    {
        IsInitialized = true;
    }
    
    if (!IsInitialized && rv != CKR_OK)
    {
        cout << "C_Initialize error. Code: " << rv << endl;
        cleanup();
        return 4;
    }
    
    CK_ULONG ulSlotCount;
    rv = p11->C_GetSlotList(FALSE, NULL_PTR, &ulSlotCount);
    if (rv != CKR_OK)
    {
        cout << "C_GetSlotList failed. Code: " << rv << endl;
        cleanup();
        return 5;
    }
    
    IsInitialized = true;
    
    return 0;
}

/**
 * manupulate echo to enter password secure
 */
void SetStdinEcho(bool enable = true)
{
#ifdef _WINDOWS
    HANDLE hStdin = GetStdHandle(STD_INPUT_HANDLE);
    DWORD mode;
    GetConsoleMode(hStdin, &mode);
    
    if( !enable )
        mode &= ~ENABLE_ECHO_INPUT;
    else
        mode |= ENABLE_ECHO_INPUT;
    
    SetConsoleMode(hStdin, mode );
    
#else
    struct termios tty;
    tcgetattr(STDIN_FILENO, &tty);
    if( !enable )
        tty.c_lflag &= ~ECHO;
    else
        tty.c_lflag |= ECHO;
    
    (void) tcsetattr(STDIN_FILENO, TCSANOW, &tty);
#endif
}

string getString(const unsigned char* buff, unsigned long buffSize)
{
    ostringstream ss;
    ss << hex << uppercase << setfill('0');
    
    for (unsigned long i = 0; i < buffSize; ++i)
    {
        ss << setw(2) << static_cast<int>(buff[i]);
    }
    
    return ss.str();
}

string getString(const vector<unsigned char>& input)
{
    
    return getString(&input[0], static_cast<unsigned long>(input.size()));
}

string getString(const size_t integer)
{
    ostringstream ss;
    ss << hex << uppercase << setfill('0');
    ss << setw(2) << integer;
    return ss.str();
}


#endif /* Utils_hpp */
