// EsmartTokenJournal.cpp : Defines the entry point for the console application.
//

#ifndef _WINDOWS
    #include <dlfcn.h>
    #include <string.h>
#endif

#include <cryptoki.h>

#ifdef __APPLE__
    #include <PCSC/wintypes.h>
    #include <PCSC/winscard.h>
#elif _WINDOWS
	#include "stdafx.h"
#else
    #include <wintypes.h>
    #include <winscard.h>
    #include <reader.h>
#endif

#include <string>
#include <vector>
#include <iostream>
#include <time.h>
#include <algorithm>
#include <gost.h>

using namespace std;

CK_FUNCTION_LIST_PTR p11 = NULL;
bool IsInitialized = false;
CK_ULONG record_size = 6;
	
#ifdef _WINDOWS
    HMODULE pkcsLib;
#else
    void *pkcsLib;
#endif

void cleanup()
{
	// Finalase library PKCS#11
	if (IsInitialized)
	{
		IsInitialized = false;
		if(p11 != NULL)
		{
			p11->C_Finalize(NULL);
			p11 = NULL;
		}
	}

	if (pkcsLib)
	{
#ifdef _WINDOWS
		FreeLibrary(pkcsLib);
#else
	    dlclose(pkcsLib);
#endif
	    pkcsLib = NULL;
	}
}

// get time diff between current time zone and UTC
int get_utc_diff()
{
    time_t timer = time(NULL);
    tm * t_utc = gmtime(&timer);
    time_t timer_utc = mktime(t_utc);
    time(&timer);
    
    return static_cast<int>(difftime(timer, timer_utc));
}

// get time since 1 Jun 2016
struct tm get_time_shift()
{
    tm t;
    t.tm_hour	= 0;
    t.tm_min	= 0;
    t.tm_sec	= 0;
    t.tm_year	= 116;
    t.tm_mon	= 0;
    t.tm_mday	= 1;
    t.tm_isdst	= 0;
    
    return t;
}

void printItem(const vector<BYTE> &buf, vector<BYTE>::size_type i)
{
    static int utc_diff = get_utc_diff();
    tm time = get_time_shift();
    time.tm_sec += utc_diff;
    
    vector<BYTE> timeBuff(4);
	reverse_copy(&buf[i], &buf[i + 4], &timeBuff[0]);
	int op_time = *(reinterpret_cast<int*>(&timeBuff[0]));
    time.tm_sec += op_time;
    // normalize time
    mktime(&time);
    
	cout << "[" << (i + 1) / record_size  << "] ";
	switch (buf[i + 4])
	{
	case 0x01:
		cout << "First perso";
		break;
	case 0x02:
		cout << "Init token";
		break;
	case 0x03:
	case 0x83:
		cout << "User PIN changed";
		break;
	case 0x04:
	case 0x84:
		cout << "SO PIN changed";
		break;
	case 0x05:
	case 0x45:
	case 0x85:
		cout << "Unsuccess attempt to authenticate by User PIN";
		break;
	case 0x06:
	case 0x46:
	case 0x86:
		cout << "Unsuccess attempt to authenticate by SO PIN";
		break;
	default:
		cout << "Unknown operation";
		break;
	}
	cout << " \t" << asctime(&time);
}

int main(int argc, char* argv[])
{
	cout << "Start with default PIN" << endl;
	BYTE pin[8];
	CK_ULONG pinLen = sizeof(pin);
	memcpy(pin, "12345678", pinLen);
	
	//////////////// Init library PKCS#11
#ifdef _WINDOWS
	pkcsLib = LoadLibrary("isbc_pkcs11_main.dll");
	if (!pkcsLib)
	{
		cout << "Error while load PKCS#11: "  << GetLastError() << endl;
		cleanup();
		return 1;
	}
#elif __APPLE__
    pkcsLib = dlopen("libisbc_pkcs11_main.dylib", RTLD_NOW);
#else
	pkcsLib = dlopen("libisbc_pkcs11_main.so", RTLD_NOW);

	if (!pkcsLib)
	{
		cout << "Error while load PKCS#11: "  << dlerror() << endl;
		cleanup();
		return 1;
	}
#endif
	
	void  (*pfoo)(CK_FUNCTION_LIST_PTR*);
	CK_RV rv;
	CK_ULONG ulSlotCount;
	
#ifdef _WINDOWS
	pfoo = (void (*)(CK_FUNCTION_LIST_PTR*)) GetProcAddress(pkcsLib, "C_GetFunctionList");
#else
	pfoo = (void (*)(CK_FUNCTION_LIST_PTR*)) dlsym(pkcsLib, "C_GetFunctionList");
#endif
	
	if(pfoo == NULL)
	{
		cout << "Can't get C_GetFunctionList symbol" << endl;
		cleanup();
		return 2;
	}

	pfoo(&p11);
	if(p11 == NULL)
	{
		cout << "C_GetFunctionList error" << endl;
		cleanup();
		return 3;
	}

	rv = p11->C_Initialize(NULL);
	if(rv == CKR_CRYPTOKI_ALREADY_INITIALIZED)
	{
		IsInitialized = true;
	}

	if (!IsInitialized && rv != CKR_OK)
	{
	    cout << "C_Initialize error. Code: " << rv << endl;
		cleanup();
	    return 4;
	}

	rv = p11->C_GetSlotList(FALSE, NULL_PTR, &ulSlotCount);
	if (rv != CKR_OK)
	{
	    cout << "C_GetSlotList failed. Code: " << rv << endl;
		cleanup();
	    return 5;
	}

	IsInitialized = true;

	// open session
	CK_SESSION_HANDLE	hSession;
	CK_SLOT_ID SlotId = 0;
	rv = p11->C_OpenSession(SlotId, CKF_SERIAL_SESSION | CKF_RW_SESSION, NULL, NULL, &hSession);

	if(rv != CKR_OK)
	{
		cout << "C_OpenSession failed. Code: " << rv << endl;
		cleanup();
		return 6;
	}
	
	//Log-In the user
	rv = p11->C_Login(hSession, CKU_USER, pin, pinLen);
	if (rv != CKR_OK && rv != CKR_USER_ALREADY_LOGGED_IN)
	{
		cout << "Login failed. Code: " << rv << endl;
		cleanup();
		return 7;
	}

	cout << "Journal reading has been started" << endl;

	CK_ULONG len = 0L;
	if (p11->C_ISBC_ScribbleRead(hSession, 0, NULL, &len) != CKR_OK) {
		cout << "Reading error" << endl;
		p11->C_CloseSession(hSession);
		cleanup();
		return 8;
	}

	if (0 == len)
	{
		cout << "Journal is empty" << endl;
		p11->C_CloseSession(hSession);
		cleanup();
		return 9;
	}

	vector<BYTE> buf(len);  
	if (p11->C_ISBC_ScribbleRead(hSession, 0, &buf[0], &len) != CKR_OK)
	{
		cout << "Reading error #2" << endl;
		p11->C_CloseSession(hSession);
		cleanup();
		return 10;
	}

	for (CK_ULONG i=record_size; i < len; i += record_size)
	{
		printItem(buf, i);
	}

	// 0 index always contain date of first perso
	printItem(buf, 0);

    p11->C_CloseSession(hSession);
	cleanup();
}
