// EsmartTokenJournal.cpp : Defines the entry point for the console application.
//

#ifndef _WINDOWS
    #include <dlfcn.h>
    #include <string.h>
	#include <termios.h>
	#include <unistd.h>
#endif

#include <cryptoki.h>

#ifdef __APPLE__
    #include <PCSC/wintypes.h>
    #include <PCSC/winscard.h>
#elif _WINDOWS
	#include "stdafx.h"
#else
    #include <wintypes.h>
    #include <winscard.h>
    #include <reader.h>
#endif

#include <string>
#include <vector>
#include <iostream>
#include <fstream>
#include <sstream>
#include <time.h>
#include <algorithm>
#include <gost.h>

using namespace std;

CK_FUNCTION_LIST_PTR p11 = NULL;
bool IsInitialized = false;
	
#ifdef _WINDOWS
    HMODULE pkcsLib;
#else
    void *pkcsLib;
#endif

void cleanup()
{
	// Finalase library PKCS#11
	if (IsInitialized)
	{
		IsInitialized = false;
		if(p11 != NULL)
		{
			p11->C_Finalize(NULL);
			p11 = NULL;
		}
	}

	if (pkcsLib)
	{
#ifdef _WINDOWS
		FreeLibrary(pkcsLib);
#else
	    dlclose(pkcsLib);
#endif
	    pkcsLib = NULL;
	}
}

void SetStdinEcho(bool enable = true)
{
#ifdef _WINDOWS
    HANDLE hStdin = GetStdHandle(STD_INPUT_HANDLE); 
    DWORD mode;
    GetConsoleMode(hStdin, &mode);

    if( !enable )
        mode &= ~ENABLE_ECHO_INPUT;
    else
        mode |= ENABLE_ECHO_INPUT;

    SetConsoleMode(hStdin, mode );

#else
    struct termios tty;
    tcgetattr(STDIN_FILENO, &tty);
    if( !enable )
        tty.c_lflag &= ~ECHO;
    else
        tty.c_lflag |= ECHO;

    (void) tcsetattr(STDIN_FILENO, TCSANOW, &tty);
#endif
}

bool readFile(const string& path, vector<BYTE>& buffer)
{
	ifstream file(path.c_str(), ios::in|ios::binary|ios::ate);
	if (file.is_open())
	{
		streampos size = file.tellg();
		buffer.assign(static_cast< vector<BYTE>::size_type>(size), 0);
		file.seekg (0, ios::beg);
		file.read(reinterpret_cast<char *>(&buffer[0]), size);
		file.close();
		return true;
	}
	return false;
}

int main(int argc, char* argv[])
{
	//////////////// Get arguments
	cout << "Please enter SO PIN (Admin PIN): ";
	SetStdinEcho(false);
    string pin;
    cin >> pin;
    SetStdinEcho(true);
	cout << endl;
	vector<BYTE> pinBuff(pin.begin(), pin.end());

	cout << "Please enter path to certificate: ";
	string path;
	cin >> path;
	cout << endl;
	vector<BYTE> ceftBuffer;
	if (!readFile(path, ceftBuffer))
	{
		cout << "Can't read file: " << path << endl;
		return 1;
	}
	
	//////////////// Init library PKCS#11
#ifdef _WINDOWS
	pkcsLib = LoadLibrary("isbc_pkcs11_main.dll");
	if (!pkcsLib)
	{
		cout << "Error while load PKCS#11: "  << GetLastError() << endl;
		cleanup();
		return 1;
	}
#elif __APPLE__
    pkcsLib = dlopen("libisbc_pkcs11_main.dylib", RTLD_NOW);
#else
	pkcsLib = dlopen("libisbc_pkcs11_main.so", RTLD_NOW);

	if (!pkcsLib)
	{
		cout << "Error while load PKCS#11: "  << dlerror() << endl;
		cleanup();
		return 1;
	}
#endif
	
	void  (*pfoo)(CK_FUNCTION_LIST_PTR*);
	CK_RV rv;
	CK_ULONG ulSlotCount;
	
#ifdef _WINDOWS
	pfoo = (void (*)(CK_FUNCTION_LIST_PTR*)) GetProcAddress(pkcsLib, "C_GetFunctionList");
#else
	pfoo = (void (*)(CK_FUNCTION_LIST_PTR*)) dlsym(pkcsLib, "C_GetFunctionList");
#endif
	
	if(pfoo == NULL)
	{
		cout << "Can't get C_GetFunctionList symbol" << endl;
		cleanup();
		return 2;
	}

	pfoo(&p11);
	if(p11 == NULL)
	{
		cout << "C_GetFunctionList error" << endl;
		cleanup();
		return 3;
	}

	rv = p11->C_Initialize(NULL);
	if(rv == CKR_CRYPTOKI_ALREADY_INITIALIZED)
	{
		IsInitialized = true;
	}

	if (!IsInitialized && rv != CKR_OK)
	{
	    cout << "C_Initialize error. Code: " << rv << endl;
		cleanup();
	    return 4;
	}

	rv = p11->C_GetSlotList(FALSE, NULL_PTR, &ulSlotCount);
	if (rv != CKR_OK)
	{
	    cout << "C_GetSlotList failed. Code: " << rv << endl;
		cleanup();
	    return 5;
	}

	IsInitialized = true;

	CK_SLOT_ID SlotId = 0;
	if (ulSlotCount > 1)
	{
		vector<CK_SLOT_ID> pSlotList(ulSlotCount);
		CK_ULONG validSlot = 0;
		stringstream ss;
		rv = p11->C_GetSlotList(CK_FALSE, &pSlotList[0], &ulSlotCount);
		if (rv == CKR_OK)
		{
			CK_SLOT_INFO slotInfo;
			CK_SLOT_ID foundSlotID;
			for (size_t i=0; i < pSlotList.size(); ++i)
			{
				foundSlotID = pSlotList[i];
				ss << "Slot " << foundSlotID  << ": ";
				rv = p11->C_GetSlotInfo(pSlotList[i], &slotInfo);
				if (rv == CKR_OK)
				{
					// drop spaces at the end
					int i = sizeof(slotInfo.manufacturerID);
					while (slotInfo.manufacturerID[i-1]==' ' && i > 0)
					{
						slotInfo.manufacturerID[i-1]='\0';
						i--;
					}
					i = sizeof(slotInfo.slotDescription);
					while (slotInfo.slotDescription[i-1]==' ' && i > 0)
					{
						slotInfo.slotDescription[i-1]='\0';
						i--;
					}

					
					ss << slotInfo.manufacturerID  << " (" << slotInfo.slotDescription << ")" << endl;
					if ((slotInfo.flags & CKF_TOKEN_PRESENT) == CKF_TOKEN_PRESENT)
					{
						++validSlot;
					}
				}
				else
				{
					cout << "C_GetSlotInfo failed. Code: " << rv << endl;
					cleanup();
					return 6;
				}
			}
		}
		else
		{
			cout << "C_GetSlotInfo failed. Code: " << rv << endl;
			cleanup();
			return 7;
		}

		if (validSlot > 1)
		{
			cout << ss.str() << "There are some slots. Please select one: ";
			string slotId;
			cin >> slotId;
			SlotId = atol(slotId.c_str());

			if (SlotId <= 0 || SlotId > ulSlotCount)
			{
				cout << "Invalid slot " << slotId << endl;
				cleanup();
				return 8;
			}
		}
	}

	// open session
	CK_SESSION_HANDLE	hSession;
	rv = p11->C_OpenSession(SlotId, CKF_SERIAL_SESSION | CKF_RW_SESSION, NULL, NULL, &hSession);

	if(rv != CKR_OK)
	{
		cout << "C_OpenSession failed. Code: " << rv << endl;
		cleanup();
		return 9;
	}
	
	//Log-In the user
	rv = p11->C_Login(hSession, CKU_SO, &pinBuff[0], static_cast<ULONG>(pinBuff.size()));
	if (rv != CKR_OK && rv != CKR_USER_ALREADY_LOGGED_IN)
	{
		cout << "Login failed. Code: " << rv << endl;
		cleanup();
		return 10;
	}

	cout << "Loading has been started" << endl;

	/* initialize random seed: */
	srand(static_cast<unsigned int>(time(NULL)));
	/* generate secret number between 1 and 10: */
	int rnd = rand();

    CK_OBJECT_CLASS obj_class = CKO_CERTIFICATE;
    CK_CERTIFICATE_TYPE certType = CKC_X_509;
	stringstream ss;
	ss << rnd;
	string label = ss.str();
	vector<CK_BYTE> labelBuff(label.begin(), label.end());
    CK_BBOOL _true = CK_TRUE;
    CK_ATTRIBUTE certificateTemplate[] = {
        {CKA_CLASS, &obj_class, sizeof(obj_class)},
        {CKA_CERTIFICATE_TYPE, &certType, sizeof(certType)},
        {CKA_TOKEN, &_true, sizeof(_true)},
		{CKA_LABEL, &labelBuff[0], static_cast<ULONG>(labelBuff.size())},
        {CKA_TRUSTED, &_true, sizeof(_true)},
        {CKA_ID, reinterpret_cast<CK_BYTE_PTR>(&rnd), static_cast<ULONG>(sizeof(rnd))},
		{CKA_VALUE, &ceftBuffer[0], static_cast<ULONG>(ceftBuffer.size())}
    };
    
    CK_OBJECT_HANDLE hCertificate;
    rv = p11->C_CreateObject(hSession, certificateTemplate, 7, &hCertificate);
    if (rv != CKR_OK)
    {
        cout << "C_CreateObject for certificate failed. Code: " << rv << endl;
        p11->C_CloseSession(hSession);
        cleanup();
        return 11;
    }
    
    cout << "Certificate has been loaded with label: " << label << endl;

    p11->C_CloseSession(hSession);
	cleanup();
}
