/**
 *  ESMART Token v3.1 - PKCS#11 Sample
 *
 *  Description: Shows token initialization (formatting), changing PIN and
 *               listing token objects
 *
 *  $Author: Alexander Revo
 *  $Date: 14.04.2014
 *  $Id: 3513-02-000
 *
 *  Copyright 2014 by ISBC Group
 *  All rights reserved.
 */

#include <iostream>
#include <string>
#include <exception>
#include "BasicOperator.h"
#include "..\common\exception_handler.h"

using namespace std;

int main()
{
	try
	{
		// Create instanse of token operator object
		// This would load PKCS DLL and initialize PKCS library
		// see CTokenOperator constructor
		CBasicOperator token;
		// If no tokens are inserted ask user to insert one
		while(!token.SelectSlot())
		{
			cout << "No tokens available. Please insert a token and press "
				"ENTER to continue or press CTRL+C to exit\r\n";
			cin.get();
		}
		for(bool exit = false; !exit; )
		{
			int choice;
			for(bool is_valid_choice = false; !is_valid_choice; )
			{
				cout << "\r\nSelect action:\r\n"
						"[0] Format token\r\n"
						"[1] Change PIN\r\n"
						"[2] List token objects\r\n"
						"[3] Exit\r\n"
						"> ";
				cin >> choice;
				if(choice <= 3 && choice >= 0)
				{
					is_valid_choice = true;
				}
			}
			switch(choice)
			{
			case 0:
				token.FormatToken();
				break;
			case 1:
				token.ChangePin();
				break;
			case 2:
				token.GetObjects();
				break;
			default: // handles exit case
				exit = true;
				break;
			}
		}
	}
	HANDLE_EXCEPTIONS()

	return 0;
}

