/**
 *  ESMART Token v3.1 - PKCS#11 Sample
 *
 *  Description: CSignatureOperator realization
 *
 *  $Author: Alexander Revo
 *  $Date: 14.04.2014
 *  $Id: 3513-03-011
 *
 *  Copyright 2014 by ISBC Group
 *  All rights reserved.
 */

#include "SignatureOperator.h"
#include <iostream>
#include <fstream>
#include <b64\encode.h>
#include <b64\decode.h>

#include <ctime>
#include <windows.h>

using namespace std;

void CSignatureOperator::GenerateRsaPair()
{
	CheckToken(); // Check if token is still present
//	Authenticate(CKF_RW_SESSION);
	CK_RV rv;

	// Key handles
	CK_OBJECT_HANDLE h_public_key;
	CK_OBJECT_HANDLE h_private_key;

	// Specify key generation parameters
	CK_MECHANISM mechanism = {CKM_RSA_PKCS_KEY_PAIR_GEN, NULL_PTR, 0};
	CK_ULONG modulus_bits = bit;
	CK_BYTE public_exponent[] = {0x01, 0x00, 0x01};
	CK_BBOOL b_true = CK_TRUE;
	CK_BYTE id[] = {123};

	// Specify public key parameters
	CK_UTF8CHAR pub_label[] = "An RSA public key object";
	CK_ATTRIBUTE public_key_template[] = {
		{CKA_ENCRYPT, &b_true, sizeof(b_true)},
		{CKA_VERIFY, &b_true, sizeof(b_true)},
		{CKA_WRAP, &b_true, sizeof(b_true)},
		{CKA_MODULUS_BITS, &modulus_bits, sizeof(modulus_bits)},
		{CKA_PUBLIC_EXPONENT, public_exponent, sizeof(public_exponent)},
		{CKA_ID, id, sizeof(id)},
		{CKA_LABEL, pub_label, sizeof(pub_label)-1}
	};

	// Specify private key parameters
	CK_UTF8CHAR priv_label[] = "An RSA private key object";
	CK_ATTRIBUTE private_key_template[] = {
		{CKA_TOKEN, &b_true, sizeof(b_true)},
		{CKA_PRIVATE, &b_true, sizeof(b_true)},
		{CKA_SENSITIVE, &b_true, sizeof(b_true)},
		{CKA_DECRYPT, &b_true, sizeof(b_true)},
		{CKA_SIGN, &b_true, sizeof(b_true)},
		{CKA_UNWRAP, &b_true, sizeof(b_true)},
		{CKA_ID, id, sizeof(id)},
		{CKA_LABEL, priv_label, sizeof(priv_label)-1}
	};

	// Generate key pair
	rv = mpFunctionList->C_GenerateKeyPair(mCurrentSession, &mechanism,
		public_key_template, 7, private_key_template, 8, &h_public_key,
		&h_private_key);
	if(CKR_OK != rv)
	{
		throw CPKCSException(rv, "CTokenOperator::GenerateRsaPair");
	}
}

void CSignatureOperator::GenerateGost3410Pair()
{
	CheckToken(); // Check if token is still present
//	Authenticate(CKF_RW_SESSION);
	CK_RV rv;

//	start_time =  clock(); //￭￠￷￠￫￼￭￮￥ ￢￰￥￬￿

	// Key handles
	CK_OBJECT_HANDLE h_public_key;
	CK_OBJECT_HANDLE h_private_key;

	// Specify key generation parameters
	CK_MECHANISM mechanism = {CKM_GOSTR3410_KEY_PAIR_GEN, NULL_PTR, 0};
	mechanism.mechanism = CKM_GOSTR3410_KEY_PAIR_GEN;
	CK_BBOOL b_true = CK_TRUE;
	CK_OBJECT_CLASS pubkey_class = CKO_PUBLIC_KEY;
	CK_OBJECT_CLASS privkey_class = CKO_PRIVATE_KEY;
	CK_BYTE id[] = {123}; //￭￥ ￡￻￫￮

	// Specify public key parameters
	CK_UTF8CHAR pub_label[] = "An GOST P34.10-2001 public key object";
	CK_ATTRIBUTE public_key_template[20] = 
	{
		{CKA_CLASS, &pubkey_class, sizeof(pubkey_class)},
		{CKA_TOKEN, &b_true, sizeof(b_true)},
		{CKA_ENCRYPT, &b_true, sizeof(b_true)},
		{CKA_VERIFY, &b_true, sizeof(b_true)},
		{CKA_WRAP, &b_true, sizeof(b_true)},
		{CKA_ID, id, sizeof(id)},							//￭￥ ￡￻￫￮
	//	{CKA_LABEL, pub_label, sizeof(pub_label)-1}			//￭￥ ￡￻￫￮
	};
	int n_pubkey_attr = 6;

	// Specify private key parameters
	CK_UTF8CHAR priv_label[] = "An GOST P34.10-2001 private key object";
	CK_ATTRIBUTE private_key_template[20] = {
		{CKA_CLASS, &privkey_class, sizeof(privkey_class)},
		{CKA_TOKEN, &b_true, sizeof(b_true)},
		{CKA_PRIVATE, &b_true, sizeof(b_true)},
		{CKA_SENSITIVE, &b_true, sizeof(b_true)},
		{CKA_DECRYPT, &b_true, sizeof(b_true)},
		{CKA_SIGN, &b_true, sizeof(b_true)},
		{CKA_UNWRAP, &b_true, sizeof(b_true)},
		{CKA_ID, id, sizeof(id)},							//￭￥ ￡￻￫￮
	//	{CKA_LABEL, priv_label, sizeof(priv_label)-1}		//￭￥ ￡￻￫￮
	};
	int n_privkey_attr = 8;



	// Generate key pair
	rv = mpFunctionList->C_GenerateKeyPair(mCurrentSession, &mechanism,
		public_key_template, n_pubkey_attr, private_key_template, n_privkey_attr, &h_public_key,
		&h_private_key);


	if(CKR_OK != rv)
	{
		throw CPKCSException(rv, "CTokenOperator::GenerateGost3410Pair");
	}


	// Logout
//	rv = mpFunctionList->C_Logout(mCurrentSession); 
}

void CSignatureOperator::GenerateCSR()
{
	CheckToken(); // Check if token is still present
	Authenticate(CKS_RW_USER_FUNCTIONS);
	CK_RV rv;
	
	// Find key objects on token
	CK_OBJECT_HANDLE h_public_key = FindPublicKey();
	CK_OBJECT_HANDLE h_private_key = FindPrivateKey();

	// Certificate distinguished name
	char* dn[] = {
		"CN", "name",
		"C", "RU"
	};

	// Certificate attributes
	char* attr[] = {"challengePassword", "1234"};

	// Certificate extra attributes
	char* ext[] = {"keyUsage", "digitalSignature,keyEncipherment", "1"};

	// C_ISBC_CreateCSR needs dn, attr and ext to be provided as
	// a null-teminated contiguous array, with fields separated
	// by a null character ('\0').
	// For example, 'dn' should be provided as "CN\0name\0C\0RU\0"

	// Make contiguous dn, attr and ext
	vector<CK_CHAR> dntemp   = MakeContiguous(dn, _countof(dn));
	vector<CK_CHAR> attrtemp = MakeContiguous(attr, _countof(attr));
	vector<CK_CHAR> exttemp  = MakeContiguous(ext, _countof(ext));

	// IMPORTANT: The following three lengths are to be equal to number of
	// field values, not number of field lines or total fields!!!
	CK_ULONG dn_length		   = 2;
	CK_ULONG attributes_length = 1;
	CK_ULONG extensions_length = 1;

	// Get CSR length
	CK_ULONG csr_length;
	rv = mpFunctionList->C_ISBC_CreateCSR(mCurrentSession, h_public_key,
		&dntemp[0], dn_length, NULL, &csr_length, h_private_key, &attrtemp[0],
		attributes_length, &exttemp[0], extensions_length);
	if (CKR_OK != rv)
	{
		throw CPKCSException(rv, "CTokenOperator::GenerateCSR");
	}

	// Generate CSR
	vector<CK_BYTE> buffer(csr_length);
	rv = mpFunctionList->C_ISBC_CreateCSR(mCurrentSession, h_public_key,
		&dntemp[0], dn_length, &buffer[0], &csr_length, h_private_key,
		&attrtemp[0], attributes_length, &exttemp[0], extensions_length);
	if (CKR_OK != rv)
	{
		throw CPKCSException(rv, "CTokenOperator::GenerateCSR");
	}

	// Logout
	rv = mpFunctionList->C_Logout(mCurrentSession);
	if(CKR_OK != rv)
	{
		throw CPKCSException(rv, "CTokenOperator::GenerateCSR");
	}

	// Write CSR into file (DER and Base64)
	WriteToFile(&buffer[0], buffer.size());

	cout << "CSR created. You may now proceed to "
			"https://www.e-notary.ru/request/chooseMethod/id/3/"
			" to get a test certificate for your CSR or contact your preferred"
			" certificate authority to get a non-test certificate" << endl;
}

void CSignatureOperator::LoadCertificate()
{
	CheckToken(); // Check if token is still present
	cout << "Obtaining existing PIN...\r\n";
	Authenticate(CKS_RW_USER_FUNCTIONS);

	CK_RV rv;

	// Read certificate from file
	vector<CK_CHAR> certificate;
	for(bool is_read_successful = false; !is_read_successful; )
	{
		is_read_successful = ReadCertificateFile(AskForFilePath(
			"x.509 certificate").c_str(), certificate);
	}

	// Create certificate object on token
	CK_BYTE id[] = { 123 };
	CK_CHAR label[] = {"Test x.509 certificate"};
	rv = mpFunctionList->C_ISBC_ImportX509Certificate(mCurrentSession, label,
		id, sizeof(id), &certificate[0], certificate.size());
	if(CKR_OK != rv)
	{
		throw CPKCSException(rv, "CTokenOperator::LoadCertificate");
	}

	// Logout
	rv = mpFunctionList->C_Logout(mCurrentSession);
	if(CKR_OK != rv)
	{
		throw CPKCSException(rv, "CTokenOperator::LoadCertificate");
	}
}

void CSignatureOperator::SignAndVerifyWithCertificate()
{
	CheckToken(); // Check if token is still present
	Authenticate();
	CK_RV rv;

	// Find certificate object on token
	CK_OBJECT_HANDLE h_certificate = FindCertificate();

	// Make random data to sign
	CK_BYTE data[32];
	for(int i = 0; i < 32; i++)
	{
		data[i] = static_cast<CK_BYTE>(rand());
	}
	
	// Get signature length
	CK_ULONG signature_length = 0;
	rv = mpFunctionList->C_ISBC_pkcs7Sign(mCurrentSession, data, sizeof(data),
		h_certificate, NULL_PTR, &signature_length, NULL,
		NULL_PTR, 0, 0);
	if(CKR_OK != rv)
	{
		throw CPKCSException(rv, "CTokenOperator::SignData");
	}

	// Allocate memory to hold signature
	vector<CK_BYTE> signature(signature_length);

	// Sign data and get signature
	rv = mpFunctionList->C_ISBC_pkcs7Sign(mCurrentSession, data, sizeof(data),
		h_certificate, &signature[0], &signature_length, NULL,
		NULL_PTR, 0, 0);
	if(CKR_OK != rv)
	{
		throw CPKCSException(rv, "CTokenOperator::SignData");
	}
	cout << "Data signed successfully." << endl;

	// Verify signature
	rv = mpFunctionList->C_ISBC_pkcs7Verify(&signature[0], signature.size(),
		data, sizeof(data));
	switch(rv)
	{
	case CKR_OK:
		cout << "Data verification successful." << endl;
		break;
	case CKR_SIGNATURE_INVALID:
		cout << "Data verification failed: "
				"signature doesn't match" << endl;
		break;
	case CKR_SIGNATURE_LEN_RANGE:
		cout << "Data verification failed: "
				"signature length doesn't match" << endl;
		break;
	default:
		throw CPKCSException(rv, "CTokenOperator::SignData");
		break;
	}

	// Logout from token
	rv = mpFunctionList->C_Logout(mCurrentSession);
	if(CKR_OK != rv)
	{
		throw CPKCSException(rv, "CTokenOperator::SignData");
	}
}

void CSignatureOperator::SignAndVerifyWithKey()
{
	CheckToken(); // Check if token is still present
	//Authenticate();

	CK_RV rv;
	
	

	// Make random data to sign
	CK_BYTE data[32];
	for(int i = 0; i < 32; i++)
	{
		data[i] = static_cast<CK_BYTE>(rand());
	}
	
	// Find private key object on token
	CK_OBJECT_HANDLE h_private_key = FindPrivateKey();

	// Specify signing mechanism
	CK_MECHANISM mechanism = {CKM_RSA_PKCS, NULL_PTR, 0};
	CK_ULONG signature_length = 0;

	// Initialize signing process
	rv = mpFunctionList->C_SignInit(mCurrentSession, &mechanism,
		h_private_key);
	if(CKR_OK != rv)
	{
		throw CPKCSException(rv, "CTokenOperator::SignData");
	}
	// Get signature length
	rv = mpFunctionList->C_Sign(mCurrentSession, data, sizeof(data), NULL_PTR,
		&signature_length);
	if(CKR_OK != rv)
	{
		throw CPKCSException(rv, "CTokenOperator::SignData");
	}

	// Allocate memory to hold signature
	vector<CK_BYTE> signature(signature_length);

	// Sign data and get signature
	rv = mpFunctionList->C_Sign(mCurrentSession, data, sizeof(data),
		&signature[0], &signature_length);
	if(CKR_OK != rv)
	{
		throw CPKCSException(rv, "CTokenOperator::SignData");
	}


	// Find public key object on token
	CK_OBJECT_HANDLE h_public_key = FindPublicKey();
	
	// Initialize verification process
	rv = mpFunctionList->C_VerifyInit(mCurrentSession, &mechanism,
		h_public_key);
	if(CKR_OK != rv)
	{
		throw CPKCSException(rv, "CTokenOperator::SignData");
	}



	// Verify signature
	rv = mpFunctionList->C_Verify(mCurrentSession, data, sizeof(data),
		&signature[0], signature_length);
	switch(rv)
	{
	case CKR_OK:
		//cout << "Data verification successful" << endl;				
		break;
	case CKR_SIGNATURE_INVALID:
		cout << "Data verification failed: "
				"signature doesn't match" << endl;
		break;
	case CKR_SIGNATURE_LEN_RANGE:
		cout << "Data verification failed: "
				"signature length doesn't match" << endl;
		break;
	default:
		throw CPKCSException(rv, "CTokenOperator::SignData");
		break;
	}
}

void CSignatureOperator::WriteToFile(CK_BYTE_PTR pData,
									 const CK_ULONG ulDataLength)
{
	const char* file_name = "csr_der.dat";

	ofstream file(file_name, ios_base::out | ios_base::binary);
	if(!file)
	{
		string message = "couldn't open file \"";
		message += file_name;
		message += "\"";
		throw std::exception(message.c_str());
	}
	// Write CSR into file (DER)
	file.write(reinterpret_cast<const char*>(pData), ulDataLength);
	file.close();

	ifstream file_in(file_name, ios_base::binary | ios_base::in);
	if(!file_in)
	{
		string message = "couldn't open file \"";
		message += file_name;
		message += "\"";
		throw std::exception(message.c_str());
	}
	ofstream file_out("csr_base64.txt", ios_base::binary | ios_base::out);
	if(!file_out)
	{
		string message = "couldn't open file \"csr_base64.txt\"";
		throw std::exception(message.c_str());
	}
	// Encode buffer in Base64
	base64::encoder b64_encoder;
	// Write CSR into file (Base64)
	b64_encoder.encode(file_in, file_out);
}

bool CSignatureOperator::ReadCertificateFile(const char* pFilePath,
										 std::vector<CK_BYTE>& rCertificate)
{
	if(!rCertificate.empty())
	{
		throw std::exception("CTokenOperator::ReadCertificateFile :"
							 " rCertificate is not empty");
	}
	ifstream file(pFilePath, ios_base::in | ios_base::binary);
	if(!file.good())
	{
		cout << "Couldn't open file \'" << pFilePath << "\"\r\nTry again.\r\n";
		return false;
	}
	vector<CK_BYTE> temp(65536);
	string cert_string;
	while(!file.eof())
	{
		file.read(reinterpret_cast<char*>(&temp[0]), temp.size());
		cert_string.append(temp.begin(), temp.begin() + file.gcount());
	}

	// Extract Base64 encoded certificate from .cer file into 'temp'
	char cert_begin[]	= "-----BEGIN CERTIFICATE-----";
	char cert_end[]		= "-----END CERTIFICATE-----";
	CK_ULONG cert_start	= cert_string.find(cert_begin)
		+ strlen(cert_begin) + 1;
	temp.assign(cert_string.begin() + cert_start,
		cert_string.begin() + cert_string.find(cert_end) - 1);
	
	// Decode Base64 encoded certificate from 'temp' into 'rCertificate'
	rCertificate.resize(temp.size(), 0);
	base64::base64_decodestate s;
    base64_init_decodestate(&s);
	int decoded_len = base64_decode_block(reinterpret_cast<char*>(&temp[0]),
		temp.size(), reinterpret_cast<char*>(&rCertificate[0]), &s);

	// Set the right size for rCertificate
	rCertificate.resize(decoded_len);

	return true;
}

std::string CSignatureOperator::AskForFilePath(const std::string& rPathToWhat)
{
	cout << "Please enter path to " << rPathToWhat << "\r\n> ";
	char buffer[260];
	ZeroMemory(buffer, 260);
	// Get filepath from user
	while(0 == strlen(buffer))
	{
		cin.getline(buffer, 260);
	}
	string result(buffer);
	return result;
}

CK_OBJECT_HANDLE CSignatureOperator::FindObject(CK_ATTRIBUTE_PTR pAttributes,
											const CK_ULONG ulAttrNumber)
{
	// Initialize search
	CK_RV rv = mpFunctionList->C_FindObjectsInit(mCurrentSession,
		pAttributes, ulAttrNumber);
	if (CKR_OK != rv)
	{
		throw CPKCSException(rv, "CTokenOperator::SignData");
	}

	// Find object
	CK_OBJECT_HANDLE h_result;
	CK_ULONG ul_object_count = 0;
	rv = mpFunctionList->C_FindObjects(mCurrentSession, &h_result, 1,
		&ul_object_count);
	if (CKR_OK != rv)
	{
		throw CPKCSException(rv, "CTokenOperator::SignData");
	}

	// Check if object has actually been found
	else if (0 == ul_object_count)
	{
		throw std::exception("Certificate not found");
	}

	// Terminate search
	mpFunctionList->C_FindObjectsFinal(mCurrentSession);

	return h_result;
}

CK_OBJECT_HANDLE CSignatureOperator::FindPublicKey()
{
	// Fill out search template for public key
	CK_BYTE public_key_id[] = { 123 };
	CK_OBJECT_CLASS pubkey_class = CKO_PUBLIC_KEY;
	CK_ATTRIBUTE public_key_template[] = {
		{CKA_CLASS, &pubkey_class, sizeof(pubkey_class)},
		{CKA_ID, reinterpret_cast<CK_VOID_PTR>(public_key_id),
			sizeof(public_key_id)}
	};
	CK_ULONG pubkey_attr_number = 2;

	return FindObject(public_key_template, pubkey_attr_number);
}

CK_OBJECT_HANDLE CSignatureOperator::FindPrivateKey()
{
	// Fill out search template for private key
	CK_BYTE private_key_id[] = { 123 };
	CK_OBJECT_CLASS privkey_class = CKO_PRIVATE_KEY;
	CK_ATTRIBUTE private_key_template[2] = {
		{CKA_CLASS, &privkey_class, sizeof(privkey_class)},
		{CKA_ID, reinterpret_cast<CK_VOID_PTR>(private_key_id),
			sizeof(private_key_id)}
	};
	CK_ULONG n_privkey_attr = 2;

	return FindObject(private_key_template, n_privkey_attr);
}

CK_OBJECT_HANDLE CSignatureOperator::FindCertificate()
{
	// Fill out search template for certificate
	CK_OBJECT_CLASS cert_class = CKO_CERTIFICATE;
	CK_BYTE cert_id[] = { 123 };
	CK_ATTRIBUTE cert_template[2] = {
		{CKA_CLASS, &cert_class, sizeof(cert_class)},
		{CKA_ID, reinterpret_cast<CK_VOID_PTR>(cert_id), sizeof(cert_id)}
	};
	CK_ULONG n_cert_attr = 2;

	return FindObject(cert_template, n_cert_attr);
}

std::vector<CK_CHAR> CSignatureOperator::MakeContiguous(char** pData,
													const size_t size)
{
	vector<CK_CHAR> result;

	DWORD dwoffset = 0;
	// Put all data from arrays in 'data' into 'result'
	for(size_t i = 0; i < size; i++)
	{
		size_t len = strlen(pData[i]) + 1;
		result.resize(result.size() + len);
		memcpy(&result[0]+dwoffset, pData[i], len);
		dwoffset += len;
	}

	return result;
}



void CSignatureOperator::DeleteRsaPair()
{
	CK_RV rv;
	
	CK_OBJECT_HANDLE h_private_key = FindPrivateKey();
	CK_OBJECT_HANDLE h_public_key = FindPublicKey();

	rv =  mpFunctionList->C_DestroyObject(mCurrentSession, h_public_key);
	rv =  mpFunctionList->C_DestroyObject(mCurrentSession, h_private_key);
	
}

void CSignatureOperator::test ()
{

			for ( int i = 0; i < TESTCOUNT; i++)
			{
				
				this->GenerateRsaPair();
				this->SignAndVerifyWithKey();
				this->DeleteRsaPair();

				cout << "operation	= " << i << endl;
			}	


}
void CSignatureOperator::logout ()
{
	CK_RV rv;
	rv = mpFunctionList->C_Logout(mCurrentSession);
}
void CSignatureOperator::SignAndVerifyWithGostKey()
{
	CheckToken(); // Check if token is still present
	//Authenticate();

	CK_RV rv;
	
	

	// Make random data to sign
	CK_BYTE data[32];
	for(int i = 0; i < 32; i++)
	{
		data[i] = static_cast<CK_BYTE>(rand());
	}
	
//			start_time =  clock(); //￭￠￷￠￫￼￭￮￥ ￢￰￥￬￿

	// Find private key object on token
	CK_OBJECT_HANDLE h_private_key = FindPrivateKey();

	// Specify signing mechanism
	CK_MECHANISM mechanism = {CKM_GOSTR3410, NULL_PTR, 0};
	CK_ULONG signature_length = 0;

	// Initialize signing process
	rv = mpFunctionList->C_SignInit(mCurrentSession, &mechanism,
		h_private_key);

	if(CKR_OK != rv)
	{
		throw CPKCSException(rv, "CTokenOperator::SignData1");
	}
	
	// Get signature length
	rv = mpFunctionList->C_Sign(mCurrentSession, data, sizeof(data), NULL_PTR,
		&signature_length);
	if(CKR_OK != rv)
	{
		throw CPKCSException(rv, "CTokenOperator::SignData2");
	}

	// Allocate memory to hold signature
	vector<CK_BYTE> signature(signature_length);



	// Sign data and get signature
	rv = mpFunctionList->C_Sign(mCurrentSession, data, sizeof(data),
		&signature[0], &signature_length);



	if(CKR_OK != rv)
	{
		throw CPKCSException(rv, "CTokenOperator::SignData3");
	}


	// Find public key object on token
	CK_OBJECT_HANDLE h_public_key = FindPublicKey();
	
	// Initialize verification process
	rv = mpFunctionList->C_VerifyInit(mCurrentSession, &mechanism,
		h_public_key);
	if(CKR_OK != rv)
	{
		throw CPKCSException(rv, "CTokenOperator::SignData4");
	}



	// Verify signature
	rv = mpFunctionList->C_Verify(mCurrentSession, data, sizeof(data),
		&signature[0], signature_length);

	
	switch(rv)
	{
	case CKR_OK:
//		cout << "Data verification successful" << endl;				
		break;
	case CKR_SIGNATURE_INVALID:
		cout << "Data verification failed: "
				"signature doesn't match" << endl;
		break;
	case CKR_SIGNATURE_LEN_RANGE:
		cout << "Data verification failed: "
				"signature length doesn't match" << endl;
		break;
	default:
		throw CPKCSException(rv, "CTokenOperator::SignData");
		break;
	}
}
void CSignatureOperator::TestGost ()
{

			for ( int i = 0; i < TESTCOUNT; i++)
			{
				this->GenerateGost3410Pair();
				this->SignAndVerifyWithGostKey();
				this->DeleteRsaPair();
				cout << "operation	= " << i << endl;
			}	

}