/**
 *  ESMART Token v3.1 - PKCS#11 Sample
 *
 *  Description: Shows RSA key pair generation, CSR generation for key pair,
 *               loading of certificate onto token, signature creation and
 *               verification using certificate.
 *
 *  $Author: Alexander Revo
 *  $Date: 14.04.2014
 *  $Id: 3513-03-000
 *
 *  Copyright 2014 by ISBC Group
 *  All rights reserved.
 */

#include <iostream>
#include <string>
#include <exception>
#include "SignatureOperator.h"
#include "..\common\exception_handler.h"

#include <ctime>
#include <conio.h>


using namespace std;

int main()
{
	try
	{
		// Create instanse of token operator object
		// This would load PKCS DLL and initialize PKCS library
		// see CTokenOperator constructor
		CSignatureOperator token;
		// If no tokens are inserted, ask user to insert one
		while(!token.SelectSlot())
		{
			cout << "No tokens available. Please insert a token and"
				    " press ENTER to continue or press CTRL+C to exit\r\n";
			cin.get();
		}
		for(bool exit = false; !exit; )
		{
			int choice;
			for(bool is_valid_choice = false; !is_valid_choice; )
			{
				cout << "\r\nSelect action:\r\n"
						"[0] 512\r\n"
						"[1] 1024\r\n"
						"[2] 2048\r\n"
						"[3] 4096\r\n"
						"[4] ALL\r\n"
						"[5] GOST\r\n"
						"[6] Exit\r\n"
						"> ";
				cin >> choice;
				if(choice <= 6 && choice >= 0)
				{
					is_valid_choice = true;
				}
			}
			switch(choice)
			{
			case 0:
				cout << "----------------------------   512 bit   ----------------------------- " << endl;
				token.Authenticate(CKF_RW_SESSION);
				token.bit = 512;
				token.test ();
			case 1:
				cout << "----------------------------   1024 bit  ----------------------------- " << endl;
				token.Authenticate(CKF_RW_SESSION);
				token.bit = 1024;
				token.test ();
				token.logout ();
			case 2:
				cout << "----------------------------   2048 bit  ----------------------------- " << endl;
				token.Authenticate(CKF_RW_SESSION);
				token.bit = 2048;
				token.test ();
				token.logout ();
			case 3:
				cout << "----------------------------   4096 bit  ----------------------------- " << endl;
				token.Authenticate(CKF_RW_SESSION);
				token.bit = 4096;
				token.test ();
				token.logout ();
			case 4:
				token.Authenticate(CKF_RW_SESSION);
				cout << "----------------------------   512 bit   ----------------------------- " << endl;
				token.bit = 512;
				token.test ();
				cout << "----------------------------   1024 bit  ----------------------------- " << endl;
				token.bit = 1024;
				token.test ();
				cout << "----------------------------   2048 bit  ----------------------------- " << endl;
				token.bit = 2048;
				token.test ();
				cout << "----------------------------   4096 bit  ----------------------------- " << endl;
				token.bit = 4096;
				token.test ();
				token.logout ();
			case 5:
				token.Authenticate (CKF_RW_SESSION);
				cout << "----------------------------		GOST	  ----------------------------- " << endl;
				token.TestGost ();
				token.logout ();
			default: // handles exit case
				exit = true;
				break;
			}
		}
	}
	HANDLE_EXCEPTIONS()
	
	getch();
	return 0;
}
