/**
 *  ESMART Token v3.1 - PKCS#11 Sample
 *
 *  Description: Provides interface to work with token
 *
 *  $Author: Alexander Revo
 *  $Date: 14.04.2014
 *  $Id: 3513-01-010
 *
 *  Copyright 2014 by ISBC Group
 *  All rights reserved.
 */

// Throws CPKCSException, std::exception

#include <windows.h>
#include <string>
#include <vector>
#include "pkcs_definitions.h"
#include "PKCSException.h"

#pragma once

class CTokenOperator
{
public: // CTORs
	CTokenOperator();
	virtual ~CTokenOperator();
private: // CTORs
	CTokenOperator(const CTokenOperator&) {};
public: // Interface
	bool SelectSlot(); // returns false if no tokens are available
	void Authenticate(CK_FLAGS flags = NULL);
	bool Authenticate(const std::vector<CK_UTF8CHAR>& rPin,
					  CK_FLAGS flags = NULL);
protected: // Member functions
	std::vector<CK_SLOT_ID> GetSlotsWithToken();
	CK_SLOT_ID AskForSlot(const std::vector<CK_SLOT_ID>& rSlots);
	void CheckToken(); // throws exception if no token is inserted
	std::string GetSlotName(const CK_SLOT_ID slotId);
	void AskForPin(std::vector<CK_UTF8CHAR>& rBuffer);
private: // Initialization
	std::wstring GetLibraryPath();
	void LoadPkcsDll();
	void InitPkcsFunctionList();
protected: // Data members
	HINSTANCE mPkcsDll;
	CK_FUNCTION_LIST_PTR mpFunctionList;
	CK_SLOT_ID mCurrentSlot;
	CK_SESSION_HANDLE mCurrentSession;
};
