/**
 *  ESMART Token v3.1 - PKCS#11 Sample
 *
 *  Description: Defines exception handling macro
 *
 *  $Author: Alexander Revo
 *  $Date: 14.04.2014
 *  $Id: 3513-01-040
 *
 *  Copyright 2014 by ISBC Group
 *  All rights reserved.
 */

#define HANDLE_EXCEPTIONS() catch(std::exception &ex) \
	{ \
		cout << "ERROR: std::exception caught: " << ex.what() << endl; \
	} \
	catch(CPKCSException& ex) \
	{ \
		std::string message = "PKCS ERROR in "; \
		message += ex.Where(); \
		message += " : "; \
		switch (ex.What()) \
		{ \
		case CKR_OK: \
			message += "CKR_OK"; \
			break; \
		case CKR_CANCEL: \
			message += "CKR_CANCEL"; \
			break; \
		case CKR_HOST_MEMORY: \
			message += "CKR_HOST_MEMORY"; \
			break; \
		case CKR_SLOT_ID_INVALID: \
			message += "CKR_SLOT_ID_INVALID"; \
			break; \
 		case CKR_GENERAL_ERROR: \
			message += "CKR_GENERAL_ERROR"; \
			break; \
		case CKR_FUNCTION_FAILED: \
			message += "CKR_FUNCTION_FAILED"; \
			break; \
		case CKR_ARGUMENTS_BAD: \
			message += "CKR_ARGUMENTS_BAD"; \
			if("CTokenOperator::SelectSlot" == ex.Where()) \
			{ \
				message += " (is PIN too long?)"; \
			} \
			break; \
		case CKR_NO_EVENT: \
			message += "CKR_NO_EVENT"; \
			break; \
		case CKR_NEED_TO_CREATE_THREADS: \
			message += "CKR_NEED_TO_CREATE_THREADS"; \
			break; \
		case CKR_CANT_LOCK: \
			message += "CKR_CANT_LOCK"; \
			break; \
		case CKR_ATTRIBUTE_READ_ONLY: \
			message += "CKR_ATTRIBUTE_READ_ONLY"; \
			break; \
		case CKR_ATTRIBUTE_SENSITIVE: \
			message += "CKR_ATTRIBUTE_SENSITIVE"; \
			break; \
		case CKR_ATTRIBUTE_TYPE_INVALID: \
			message += "CKR_ATTRIBUTE_TYPE_INVALID"; \
			break; \
		case CKR_ATTRIBUTE_VALUE_INVALID: \
			message += "CKR_ATTRIBUTE_VALUE_INVALID"; \
			break; \
		case CKR_DATA_INVALID: \
			message += "CKR_DATA_INVALID"; \
			break; \
		case CKR_DATA_LEN_RANGE: \
			message += "CKR_DATA_LEN_RANGE"; \
			break; \
		case CKR_DEVICE_ERROR: \
			message += "CKR_DEVICE_ERROR"; \
			break; \
		case CKR_DEVICE_MEMORY: \
			message += "CKR_DEVICE_MEMORY"; \
			break; \
		case CKR_DEVICE_REMOVED: \
			message += "CKR_DEVICE_REMOVED"; \
			break; \
		case CKR_ENCRYPTED_DATA_INVALID: \
			message += "CKR_ENCRYPTED_DATA_INVALID"; \
			break; \
		case CKR_ENCRYPTED_DATA_LEN_RANGE: \
			message += "CKR_ENCRYPTED_DATA_LEN_RANGE"; \
			break; \
		case CKR_FUNCTION_CANCELED: \
			message += "CKR_FUNCTION_CANCELED"; \
			break; \
		case CKR_FUNCTION_NOT_PARALLEL: \
			message += "CKR_FUNCTION_NOT_PARALLEL"; \
			break; \
 		case CKR_FUNCTION_NOT_SUPPORTED: \
			message += "CKR_FUNCTION_NOT_SUPPORTED"; \
			break; \
 		case CKR_KEY_HANDLE_INVALID: \
			message += "CKR_KEY_HANDLE_INVALID"; \
			break; \
 		case CKR_KEY_SIZE_RANGE: \
			message += "CKR_KEY_SIZE_RANGE"; \
			break; \
		case CKR_KEY_TYPE_INCONSISTENT: \
			message += "CKR_KEY_TYPE_INCONSISTENT"; \
			break; \
		case CKR_KEY_NOT_NEEDED: \
			message += "CKR_KEY_NOT_NEEDED"; \
			break; \
		case CKR_KEY_CHANGED: \
			message += "CKR_KEY_CHANGED"; \
			break; \
		case CKR_KEY_NEEDED: \
			message += "CKR_KEY_NEEDED"; \
			break; \
		case CKR_KEY_INDIGESTIBLE: \
			message += "CKR_KEY_INDIGESTIBLE"; \
			break; \
		case CKR_KEY_FUNCTION_NOT_PERMITTED: \
			message += "CKR_KEY_FUNCTION_NOT_PERMITTED"; \
			break; \
		case CKR_KEY_NOT_WRAPPABLE: \
			message += "CKR_KEY_NOT_WRAPPABLE"; \
			break; \
		case CKR_KEY_UNEXTRACTABLE: \
			message += "CKR_KEY_UNEXTRACTABLE"; \
			break; \
 		case CKR_MECHANISM_INVALID: \
			message += "CKR_MECHANISM_INVALID"; \
			break; \
		case CKR_MECHANISM_PARAM_INVALID: \
			message += "CKR_MECHANISM_PARAM_INVALID"; \
			break; \
		case CKR_OBJECT_HANDLE_INVALID: \
			message += "CKR_OBJECT_HANDLE_INVALID"; \
			break; \
		case CKR_OPERATION_ACTIVE: \
			message += "CKR_OPERATION_ACTIVE"; \
			break; \
		case CKR_OPERATION_NOT_INITIALIZED: \
			message += "CKR_OPERATION_NOT_INITIALIZED"; \
			break; \
		case CKR_PIN_INCORRECT: \
			message += "CKR_PIN_INCORRECT"; \
			break; \
		case CKR_PIN_INVALID: \
			message += "CKR_PIN_INVALID"; \
			break; \
		case CKR_PIN_LEN_RANGE: \
			message += "CKR_PIN_LEN_RANGE"; \
			break; \
		case CKR_PIN_EXPIRED: \
			message += "CKR_PIN_EXPIRED"; \
			break; \
		case CKR_PIN_LOCKED: \
			message += "CKR_PIN_LOCKED"; \
			break; \
 		case CKR_SESSION_CLOSED: \
			message += "CKR_SESSION_CLOSED"; \
			break; \
		case CKR_SESSION_COUNT: \
			message += "CKR_SESSION_COUNT"; \
			break; \
		case CKR_SESSION_HANDLE_INVALID: \
			message += "CKR_SESSION_HANDLE_INVALID"; \
			break; \
		case CKR_SESSION_PARALLEL_NOT_SUPPORTED: \
			message += "CKR_SESSION_PARALLEL_NOT_SUPPORTED"; \
			break; \
		case CKR_SESSION_READ_ONLY: \
			message += "CKR_SESSION_READ_ONLY"; \
			break; \
		case CKR_SESSION_EXISTS: \
			message += "CKR_SESSION_EXISTS"; \
			break; \
		case CKR_SESSION_READ_ONLY_EXISTS: \
			message += "CKR_SESSION_READ_ONLY_EXISTS"; \
			break; \
		case CKR_SESSION_READ_WRITE_SO_EXISTS: \
			message += "CKR_SESSION_READ_WRITE_SO_EXISTS"; \
			break; \
		case CKR_SIGNATURE_INVALID: \
			message += "CKR_SIGNATURE_INVALID"; \
			break; \
		case CKR_SIGNATURE_LEN_RANGE: \
			message += "CKR_SIGNATURE_LEN_RANGE"; \
			break; \
		case CKR_TEMPLATE_INCOMPLETE: \
			message += "CKR_TEMPLATE_INCOMPLETE"; \
			break; \
		case CKR_TEMPLATE_INCONSISTENT: \
			message += "CKR_TEMPLATE_INCONSISTENT"; \
			break; \
		case CKR_TOKEN_NOT_PRESENT: \
			message += "CKR_TOKEN_NOT_PRESENT"; \
			break; \
		case CKR_TOKEN_NOT_RECOGNIZED: \
			message += "CKR_TOKEN_NOT_RECOGNIZED"; \
			break; \
		case CKR_TOKEN_WRITE_PROTECTED: \
			message += "CKR_TOKEN_WRITE_PROTECTED"; \
			break; \
		case CKR_UNWRAPPING_KEY_HANDLE_INVALID: \
			message += "CKR_UNWRAPPING_KEY_HANDLE_INVALID"; \
			break; \
		case CKR_UNWRAPPING_KEY_SIZE_RANGE: \
			message += "CKR_UNWRAPPING_KEY_SIZE_RANGE"; \
			break; \
		case CKR_UNWRAPPING_KEY_TYPE_INCONSISTENT: \
			message += "CKR_UNWRAPPING_KEY_TYPE_INCONSISTENT"; \
			break; \
		case CKR_USER_ALREADY_LOGGED_IN: \
			message += "CKR_USER_ALREADY_LOGGED_IN"; \
			break; \
		case CKR_USER_NOT_LOGGED_IN: \
			message += "CKR_USER_NOT_LOGGED_IN"; \
			break; \
		case CKR_USER_PIN_NOT_INITIALIZED: \
			message += "CKR_USER_PIN_NOT_INITIALIZED"; \
			break; \
		case CKR_USER_TYPE_INVALID: \
			message += "CKR_USER_TYPE_INVALID"; \
			break; \
		case CKR_USER_ANOTHER_ALREADY_LOGGED_IN: \
			message += "CKR_USER_ANOTHER_ALREADY_LOGGED_IN"; \
			break; \
		case CKR_USER_TOO_MANY_TYPES: \
			message += "CKR_USER_TOO_MANY_TYPES"; \
			break; \
		case CKR_WRAPPED_KEY_INVALID: \
			message += "CKR_WRAPPED_KEY_INVALID"; \
			break; \
		case CKR_WRAPPED_KEY_LEN_RANGE: \
			message += "CKR_WRAPPED_KEY_LEN_RANGE"; \
			break; \
		case CKR_WRAPPING_KEY_HANDLE_INVALID: \
			message += "CKR_WRAPPING_KEY_HANDLE_INVALID"; \
			break; \
		case CKR_WRAPPING_KEY_SIZE_RANGE: \
			message += "CKR_WRAPPING_KEY_SIZE_RANGE"; \
			break; \
		case CKR_WRAPPING_KEY_TYPE_INCONSISTENT: \
			message += "CKR_WRAPPING_KEY_TYPE_INCONSISTENT"; \
			break; \
		case CKR_RANDOM_SEED_NOT_SUPPORTED: \
			message += "CKR_RANDOM_SEED_NOT_SUPPORTED"; \
			break; \
		case CKR_RANDOM_NO_RNG: \
			message += "CKR_RANDOM_NO_RNG"; \
			break; \
		case CKR_DOMAIN_PARAMS_INVALID: \
			message += "CKR_DOMAIN_PARAMS_INVALID"; \
			break; \
		case CKR_BUFFER_TOO_SMALL: \
			message += "CKR_BUFFER_TOO_SMALL"; \
			break; \
		case CKR_SAVED_STATE_INVALID: \
			message += "CKR_SAVED_STATE_INVALID"; \
			break; \
		case CKR_INFORMATION_SENSITIVE: \
			message += "CKR_INFORMATION_SENSITIVE"; \
			break; \
		case CKR_STATE_UNSAVEABLE: \
			message += "CKR_STATE_UNSAVEABLE"; \
			break; \
		case CKR_CRYPTOKI_NOT_INITIALIZED: \
			message += "CKR_CRYPTOKI_NOT_INITIALIZED"; \
			break; \
		case CKR_CRYPTOKI_ALREADY_INITIALIZED: \
			message += "CKR_CRYPTOKI_ALREADY_INITIALIZED"; \
			break; \
		case CKR_MUTEX_BAD: \
			message += "CKR_MUTEX_BAD"; \
			break; \
		case CKR_MUTEX_NOT_LOCKED: \
			message += "CKR_MUTEX_NOT_LOCKED"; \
			break; \
		case CKR_NEW_PIN_MODE: \
			message += "CKR_NEW_PIN_MODE"; \
			break; \
		case CKR_NEXT_OTP: \
			message += "CKR_NEXT_OTP"; \
			break; \
		case CKR_FUNCTION_REJECTED: \
			message += "CKR_FUNCTION_REJECTED"; \
			break; \
		case CKR_VENDOR_DEFINED: \
			message += "CKR_VENDOR_DEFINED"; \
			break; \
		} \
		cout << message << endl; \
	} \
	catch(...) \
	{ \
		cout << "unknown exception caught" << endl; \
	}